/*
 * MIT License
 *
 * Copyright (c) 2023-2024 Maciej 'maQ' Kusz <maciej.kusz@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

.md-typeset .post {
    border-top: 0.05rem dotted black;
    margin-top: 1em;
    padding-top: 1em;
}

.md-typeset .post:first-of-type {
    border-top: none;
}

.md-typeset .post-extra {
    display: flex;
    flex-direction: row;
}

.md-typeset .post-extra .col {
    width: 100%;
}

.md-typeset .post-extra .col:first-of-type {
    padding-left: 0;
}

.md-typeset .post-extra .col:last-of-type {
    padding-right: 0;
    text-align: end;
}

.md-typeset .post-extra .col p {
    margin: 0 0 0 0;
    font-size: 0.9em;
}

.md-typeset .post-extra .post-tag {
    display: inline;
}

.md-typeset .post-extra .post-date {
    display: inline;
}

.md-typeset .post-link {
    text-align: end;
    border-bottom: 0.05rem dotted black;
    padding-bottom: 1em;
    margin: 1em 0 1em;
}

.md-typeset p {
    text-align: justify;
    text-justify: auto;
}
