<!--
  ~ MIT License
  ~
  ~ Copyright (c) 2023-2024 Maciej 'maQ' Kusz <maciej.kusz@gmail.com>
  ~
  ~ Permission is hereby granted, free of charge, to any person obtaining a copy
  ~ of this software and associated documentation files (the "Software"), to deal
  ~ in the Software without restriction, including without limitation the rights
  ~ to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  ~ copies of the Software, and to permit persons to whom the Software is
  ~ furnished to do so, subject to the following conditions:
  ~
  ~ The above copyright notice and this permission notice shall be included in all
  ~ copies or substantial portions of the Software.
  ~
  ~ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  ~ IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  ~ FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  ~ AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  ~ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  ~ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  ~ SOFTWARE.
  -->

{% for post in posts %}
## [{{ post.title }}]({{ site_url }}{{ config.slug }}/{{ post.slug }}/)

<!--suppress LongLine -->
<div class="post-extra">
    <div class="col">
        <p class="post-date">{{ post.date }}</p>
    </div>
    <div class="col">
    {% for tag in post.tags %}
        <a href="{{ site_url }}{{ config.slug }}/{{ config.tags_subdir }}/{{ tag }}/">#{{ tag }}</a>
    {% endfor %}
    </div>
</div>

{{ post.teaser }}

{{ post.is_short }}

<div class="post-link">
{% if post.is_teaser %}
    <a href="{{ site_url }}{{ config.slug }}/{{ post.slug }}/" title="{ post.title }">
        {{ translation.teaser_link_text }}
    </a>
{% else %}
    &nbsp;
{% endif %}
</div>

{% endfor %}
